@@ -1,43 +1,72 @@
 import copy
 import os
+import csv
 import time
+from io import StringIO
+from django.contrib import messages
 from functools import partial
-
-import cachetools
+from import_export.admin import ImportExportModelAdmin
 from django import forms
-from .models import SnortRule, SnortRuleViewArray
+from .models import SnortRule, SnortRuleViewArray, save_rule_to_s3, delete_rule_from_s3
 from .snort_templates import types_list
 from .parser import Parser
 from django.utils.encoding import smart_str
 from django.http.response import HttpResponse, HttpResponseRedirect
 from django.utils.html import mark_safe
 from django.db import transaction
-from django.views.decorators.cache import never_cache
+import suricataparser
+from snort.views import build_keyword_dict
 # Register your models here.
 from django.contrib import admin
 from django_object_actions import DjangoObjectActions
 import subprocess
-from settings.models import Setting, keywords
+from settings.models import Setting, keywords, attackGroup
 from django.shortcuts import render
 from pcaps.admin import verify_legal_pcap
+from advanced_filters.admin import AdminAdvancedFiltersMixin
+from django.core.files.storage import default_storage
+from django.core.files.base import ContentFile
+from django.conf import settings
+from datetime import datetime
 
-FIELDS = (
-    "id", "full_rule", "active", "is_template", "deleted", "admin_locked", 'name', "snort_builder", "request_ref", "main_ref", "description",
-    "group", "extra", "location", "user", 'pcap_sanity_check', "pcap_legal_check")
 
+class StoreAdminForm(forms.ModelForm):
+    ## add an extra field:
+    upfile = forms.FileField()
+
+    class Meta:
+        model = SnortRule
+        fields = "__all__"
+
+    def clean(self):
+        cleaned_data = super(StoreAdminForm, self).clean()
+
+        if "upfile" in self.changed_data:
+            ### file validation on file type etc here ..
+            ## file is valid:
+            ## next lines deal with the InMemoryUploadedFile Type
+            path = settings.MEDIA_ROOT.joinpath("___tmp___")
+            tmp = default_storage.save(path, ContentFile(cleaned_data["upfile"].read()))
+
+            ## ...
+
+BASE_FIELDS = [
+    "id", "active", "is_template", "deleted", "admin_locked", 'name', "request_ref", "main_ref", "description",
+    "extra", "location", "user"]
+FILTER_FIELDS = ("active", "is_template", "deleted", "admin_locked")
+ADVANCE_FILTER_FIELDS = tuple(i for i in BASE_FIELDS + ["content", ("pcap_sanity_check__name", "pcap_sanity_check_name"), ("pcap_legal_check__name", "pcap_legal_check_name"),("group_name", "group_name")])
+FIELDS = [
+    "id", "content", "active", "is_template", "deleted", "admin_locked", 'name', "request_ref", "main_ref", "snort_builder", "description",
+    "group", "extra", "location", "user", 'pcap_sanity_check', "pcap_legal_check", ]
+SEARCH_FIELDS = tuple(i for i in BASE_FIELDS + ["content", "pcap_sanity_check__name", "pcap_legal_check__name", "group__name"])
 BASE_BUILDER_KEY = ("action", "protocol", "srcipallow", "srcip", "srcportallow", "srcport", "direction", "dstipallow",
                     "dstportallow", "dstport")
 
 INPUT_TYPE = ("srcip" , "srcport", "dstip", "dstport")
 from django.core.cache import cache
 # todo: upload unmanaged rule file
-# todo: export to csv
 
 class SnortRuleAdminForm(forms.ModelForm):
-    class Meta:
-        model = SnortRule
-        fields = "__all__"
-
     def clean_user(self):
         return getattr(self.current_user, self.current_user.USERNAME_FIELD)
 
@@ -51,13 +80,13 @@ class SnortRuleAdminForm(forms.ModelForm):
 
     def clean_content(self):
         try:
-            parser = Parser(self.data["full_rule"])
+            parser = Parser(self.data["content"])
             parser.parse_header()
             parser.parse_options()
         except Exception as e:
             raise forms.ValidationError(e)
 
-        return self.data["full_rule"]
+        return self.data["content"]
 
     def clean_location(self):
         try:
@@ -87,7 +116,7 @@ class SnortRuleAdminForm(forms.ModelForm):
                 raise forms.ValidationError(
                     f"bad configuration setting (FORCE_SANITY_CHECK), pleas edit setting(FORCE_SANITY_CHECK) must be True or False")
         cur_rule = SnortRule()
-        cur_rule.content = self.data.get("full_rule")
+        cur_rule.content = self.data.get("content")
         cur_rule.location = self.data.get("location")
         cur_rule.group = self.instance.group
         cur_rule.id = self.data.get("id")
@@ -167,11 +196,11 @@ class SnortRuleAdminForm(forms.ModelForm):
         self.instance.deleted = False
         if not self.instance.pk and not self.errors:
             self.instance.save()
-        SnortRuleViewArray.objects.filter(snortId=None).delete()
+        # SnortRuleViewArray.objects.filter(snortId=None).delete()
         if not self.errors:
             SnortRuleViewArray.objects.filter(snortId=self.instance.id).delete()
         for key, value in self.data.items():
-            if key in FIELDS + ('csrfmiddlewaretoken', "_save"):
+            if key in FIELDS + ['csrfmiddlewaretoken', "_save"]:
                 continue
             item_type = "select"
             location_x = 0
@@ -210,10 +239,11 @@ class SnortRuleAdminForm(forms.ModelForm):
             cache.set(self.instance.id, rule_keys)
             return
         if self.cleaned_data.get("active"):
+            save_rule_to_s3(self.instance.id, self.instance.content)
             pass
             # todo: save to s3
         else:
-            pass
+            delete_rule_from_s3(self.instance.id)
             # todo: make sure it is not on prod
 
 
@@ -253,27 +283,166 @@ def validate_pcap_snort(pcaps, rule):
 
 
 @admin.register(SnortRule)
-class SnortRuleAdmin(DjangoObjectActions, admin.ModelAdmin):
+class SnortRuleAdmin(DjangoObjectActions, AdminAdvancedFiltersMixin, ImportExportModelAdmin, admin.ModelAdmin):
+    list_filter = FILTER_FIELDS  # simple list filters
+
+    # specify which fields can be selected in the advanced filter
+    # creation form
+    advanced_filter_fields = ADVANCE_FILTER_FIELDS
     change_actions = ('clone_rule',)
     # changelist_actions = ('load_template',)
     fields = FIELDS
     filter_horizontal = ('pcap_sanity_check', "pcap_legal_check")
     list_display_links = ("name",)
-    list_display = ("id", "active", "name", "group", "description", "date", "is_template")
-    search_fields = (
-    "active", 'name', "request_ref", "main_ref", "description", "group", "content", "extra", "location", "user")
+    list_display = ("id", "user", "active", "name", "group", "description", "content", "date", "is_template")
+    search_fields = SEARCH_FIELDS
     form = SnortRuleAdminForm
     actions = ['make_published']
 
+    def export_action(self, request):
+        return self.export_data(SnortRule.objects.all())
+
+    @transaction.atomic
+    def import_action(self, request):
+        errors = False
+        if request.method == 'POST':
+            snort_rules_to_save = []
+            snort_rules_options_to_save = {}
+            try:
+                csv_file = request.FILES['myfile']
+                if not csv_file.name.endswith('.csv'):
+                    messages.error(request, 'File is not CSV type')
+                    return HttpResponseRedirect("/admin/snort/snortrule/import/")
+                # if file is too large, return
+                if csv_file.multiple_chunks():
+                    messages.error(request, "Uploaded file is too big (%.2f MB)." % (csv_file.size / (1000 * 1000),))
+                    return HttpResponseRedirect("/admin/snort/snortrule/import/")
+
+                file_data = csv.DictReader(StringIO(csv_file.read().decode("utf-8")))
+                # loop over the lines and save them in db. If error , store as string and then display
+                for item in file_data:
+                    snort_rule = SnortRule()
+                    snort_rule.active = item["Active"]
+                    snort_rule.deleted = item["Deleted"]
+                    snort_rule.user = getattr(request.user, request.user.USERNAME_FIELD)
+                    snort_rule.description = item["Description"]
+                    snort_rule.extra = item["Extra"]
+                    if item.get("Group"):
+                        snort_rule.group = attackGroup.objects.get(name=item["Group"])
+                    snort_rule.name = item["Name"]
+                    if item.get("Id"):
+                        snort_rule.id = item["Id"]
+                    else:
+                        temp_id = str(time.time())
+                        snort_rule.id = "temp " + temp_id
+                    snort_rule.content = item["Rule"]
+
+                    try:
+                        resppnse = {"data": []}
+                        try:
+                            rule_parsed = suricataparser.parse_rule(item["Rule"])
+                        except:
+                            raise Exception("bad rule format")
+                        build_keyword_dict(resppnse, rule_parsed)
+                        for item_data in resppnse["data"]:
+                            temp_id = snort_rule.id
+                            snort_rules_options_to_save[temp_id] = []
+                            snort_rules_options_to_save[temp_id].append(SnortRuleViewArray(**item_data))
+                        for op in rule_parsed.options:
+                            if op.name == "msg":
+                                if snort_rule.group:
+                                    op.value = snort_rule.group.name + " "
+                                else:
+                                    op.value = ""
+                                if snort_rule.name:
+                                    op.value += snort_rule.name
+                                continue
+                            if op.name == "sid":
+                                op.value = snort_rule.id
+                                continue
+                            if op.name == "metadata":
+                                new_value = []
+                                user_applyed = False
+                                for item_metadata in op.value.data:
+                                    if item_metadata.strip("'").strip().startswith("group "):
+                                        if snort_rule.group:
+                                            new_value.append(f"group {snort_rule.group.name}")
+                                            continue
+                                    if item_metadata.strip("'").strip().startswith("name "):
+                                        new_value.append(f"name {snort_rule.name}")
+                                        continue
+                                    if item_metadata.strip("'").strip().startswith("description "):
+                                        new_value.append(f"description {snort_rule.description}")
+                                        continue
+                                    if item_metadata.strip("'").strip().startswith("employee "):
+                                        new_value.append(f"employee {snort_rule.user}")
+                                        user_applyed = False
+                                        continue
+                                    if item_metadata.strip("'").strip().startswith("document "):
+                                        snort_rule.main_ref = item_metadata.strip("'").strip().replace("document ", "")
+                                    if item_metadata.strip("'").strip().startswith("treatment "):
+                                        snort_rule.request_ref = item_metadata.strip("'").strip().replace("document ", "")
+                                    new_value.append(item_metadata)
+                                if not user_applyed:
+                                    new_value.append(f"employee {snort_rule.user}")
+                                op.value.data = new_value
+                                continue
+                        snort_rule.content = rule_parsed
+                        snort_rules_to_save.append(snort_rule)
+                    except Exception as e:
+                        import traceback
+                        traceback.print_exc()
+                        errors = True
+                        messages.error(request, f"Unable to load rule {item['Name']}. {repr(e)}")
+                        pass
+
+            except Exception as e:
+                errors = True
+                messages.error(request, "Unable to upload file. " + repr(e))
+            if not errors:
+                for rule in snort_rules_to_save:
+                    rule_id = rule.id
+                    if rule_id.startswith("temp "):
+                        rule.id = None
+                        rule_parsed = rule.content
+                        for op in rule_parsed.options:
+                            if op.name == "sid":
+                                op.value = 0
+                                break
+                        rule.content = ""
+                        rule.save()
+                        for op in rule_parsed.options:
+                            if op.name == "sid":
+                                op.value = rule.id
+                                break
+                    rule.content = rule_parsed.build_rule()
+                    rule.save()
+                    if rule.active:
+                        save_rule_to_s3(rule.id, rule.content)
+                    else:
+                        delete_rule_from_s3(rule.id)
+                    SnortRuleViewArray.objects.filter(snortId=rule.id).delete()
+                    for attr in snort_rules_options_to_save[rule_id]:
+                        attr.snortId = rule
+                        attr.save()
+                return HttpResponseRedirect("/admin/snort/snortrule/")
+            return HttpResponseRedirect("/admin/snort/snortrule/import/")
+
+        return render(request, 'html/import.html')
+
     @admin.action(description='export selected snort to csv')
     def make_published(self, request, queryset):
+        return self.export_data(queryset)
+
+    def export_data(self, queryset):
         response = HttpResponse(
             content_type='application/force-download')  # mimetype is replaced by content_type for django 1.7
-        response['Content-Disposition'] = 'attachment; filename=%s' % smart_str("snort_export.csv")
-        str_content = "active,date,deleted,description,extra,group,name,id,user,content\n"
+        response['Content-Disposition'] = 'attachment; filename=%s' % smart_str(f"SnortRule-{datetime.now()}.csv")
+        str_content = "Active,Date,Deleted,Description,Extra,Group,Name,Id,User,Rule\n"
         for snort_item in queryset:
             content = snort_item.content.replace('"', "'")
-            my_list = [snort_item.active,snort_item.date,snort_item.deleted,snort_item.description,snort_item.extra,snort_item.group,snort_item.name,snort_item.pk,snort_item.user,content]
+            my_list = [snort_item.active, snort_item.date, snort_item.deleted, snort_item.description, snort_item.extra,
+                       snort_item.group, snort_item.name, snort_item.pk, snort_item.user, content]
             for item in my_list:
                 if isinstance(item, bool):
                     str_content += str(item) + ","
@@ -294,6 +463,7 @@ class SnortRuleAdmin(DjangoObjectActions, admin.ModelAdmin):
         if 'delete_selected' in actions:
             del actions['delete_selected']
         return actions
+
     def snort_builder(self, obj):
         set_rule = cache.get(obj.id)
         if not set_rule:
@@ -304,13 +474,7 @@ class SnortRuleAdmin(DjangoObjectActions, admin.ModelAdmin):
         context = copy.deepcopy(self.context)
         context["build_items"] = set_rule
         snort_buider_section = self.snort_buider_section(context).content.decode("utf-8")
-        return mark_safe(snort_buider_section)
-
-    def full_rule(self, obj):
-        test = mark_safe(self.full_rule_js.content.decode("utf-8"))
-        rule = obj
-        full_rule = ""
-        return test
+        return mark_safe(self.full_rule_js.content.decode("utf-8") + snort_buider_section)
 
     def get_form(self, request, *args, **kwargs):
         form = super(SnortRuleAdmin, self).get_form(request, **kwargs)
@@ -326,7 +490,7 @@ class SnortRuleAdmin(DjangoObjectActions, admin.ModelAdmin):
     def clone_rule(self, request, obj: SnortRule):
         new_snort = SnortRule.objects.get(pk=obj.id)
         new_snort.pk = None
-        new_snort.template = False
+        new_snort.is_template = False
         new_snort.active = False
         new_snort.deleted = False
         new_snort.user = getattr(request.user, request.user.USERNAME_FIELD)
@@ -343,9 +507,9 @@ class SnortRuleAdmin(DjangoObjectActions, admin.ModelAdmin):
     def get_readonly_fields(self, request, obj=None):
         if obj and (obj.is_template or obj.admin_locked):
             read_only_fields = (
-            "id", "active", 'location', "user", "admin_locked", "full_rule", "snort_builder", "deleted")
+            "id", "active", 'location', "user", "admin_locked", "snort_builder", "deleted")
         else:
-            read_only_fields = ("id", 'location', "user", "admin_locked", "full_rule", "snort_builder", "deleted")
+            read_only_fields = ("id", 'location', "user", "admin_locked", "snort_builder", "deleted")
 
         return read_only_fields
 
